import { Comment } from './comment-handler';
import { Error } from './error-handler';
import { StatementType } from './syntax';
import { TokenEntry } from './token';
export type ArgumentListElement = Expression | SpreadElement;
export type ArrayExpressionElement = Expression | SpreadElement | null;
export type ArrayPatternElement = AssignmentPattern | BindingIdentifier | BindingPattern | RestElement | null;
export type BindingPattern = ArrayPattern | ObjectPattern;
export type BindingIdentifier = Identifier;
export type Declaration = AsyncFunctionDeclaration | ClassDeclaration | ExportDeclaration | FunctionDeclaration | ImportDeclaration | VariableDeclaration;
export type ExportableDefaultDeclaration = BindingIdentifier | BindingPattern | ClassDeclaration | Expression | FunctionDeclaration;
export type ExportableNamedDeclaration = AsyncFunctionDeclaration | ClassDeclaration | FunctionDeclaration | VariableDeclaration;
export type ExportDeclaration = ExportAllDeclaration | ExportDefaultDeclaration | ExportNamedDeclaration;
export type Expression = ArrayExpression | ArrowFunctionExpression | AssignmentExpression | AsyncArrowFunctionExpression | AsyncFunctionExpression | AwaitExpression | BinaryExpression | CallExpression | ClassExpression | ComputedMemberExpression | ConditionalExpression | Identifier | FunctionExpression | Literal | NewExpression | ObjectExpression | RegexLiteral | SequenceExpression | SpreadElement | StaticMemberExpression | TaggedTemplateExpression | ThisExpression | UnaryExpression | UpdateExpression | YieldExpression;
export type FunctionParameter = AssignmentPattern | BindingIdentifier | BindingPattern;
export type ImportDeclarationSpecifier = ImportDefaultSpecifier | ImportNamespaceSpecifier | ImportSpecifier;
export type ObjectExpressionProperty = Property | SpreadProperty;
export type ObjectPatternProperty = Property | RestProperty;
export type PatternParam = ArrayPatternElement | Identifier | BindingPattern | Expression | RestProperty | PropertyValue;
export type Statement = AsyncFunctionDeclaration | BreakStatement | ClassDeclaration | ContinueStatement | DebuggerStatement | DoWhileStatement | EmptyStatement | ExpressionStatement | Directive | ForStatement | ForInStatement | ForOfStatement | FunctionDeclaration | IfStatement | LabeledStatement | ReturnStatement | SwitchStatement | ThrowStatement | TryStatement | VariableDeclaration | WhileStatement | WithStatement;
export type ReinterpretedExpressionPattern = BindingPattern | AssignmentPattern | RestElement | RestProperty;
export type PropertyKey = Identifier | Literal | Expression;
export declare function isIdentifier(key: {
    type: string;
}): key is Identifier;
export declare function isLiteral(key: PropertyKey): key is Literal;
export type PropertyValue = AssignmentPattern | AsyncFunctionExpression | BindingIdentifier | BindingPattern | FunctionExpression | Expression;
export type StatementListItem = Declaration | Statement;
export interface Position {
    /** >= 1 */
    line: number;
    /** >= 0 */
    column: number;
}
export interface SourceLocation {
    source?: string | null | undefined;
    start: Position;
    end: Position;
}
export interface BaseNodeWithoutComments {
    type: string;
    loc?: SourceLocation | null | undefined;
    range?: [number, number] | undefined;
}
export interface BaseNode extends BaseNodeWithoutComments {
    leadingComments?: Comment[];
    trailingComments?: Comment[];
}
export declare class ArrayExpression implements BaseNode {
    type: StatementType;
    readonly elements: ArrayExpressionElement[];
    constructor(elements: ArrayExpressionElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isArrayExpression(p: {
    type: StatementType;
}): p is ArrayExpression;
export declare class ArrayPattern implements BaseNode {
    readonly type: StatementType;
    readonly elements: ArrayPatternElement[];
    constructor(elements: ArrayPatternElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isArrayPattern(p: {
    type: StatementType;
}): p is ArrayPattern;
export declare class ArrowFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: (FunctionParameter | PatternParam)[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    readonly rest?: Identifier;
    readonly defaults?: Expression[];
    constructor(params: (FunctionParameter | PatternParam)[], body: BlockStatement | Expression, expression: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isArrowFunctionExpression(p: {
    type: StatementType;
}): p is ArrowFunctionExpression;
export declare class AssignmentExpression implements BaseNode {
    type: StatementType;
    operator: string;
    readonly left: Expression;
    readonly right: Expression;
    constructor(operator: string, left: Expression, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isAssignmentExpression(p: {
    type: StatementType;
}): p is AssignmentExpression;
export declare class AssignmentPattern implements BaseNode {
    readonly type: StatementType;
    readonly left: BindingIdentifier | BindingPattern;
    readonly right: Expression;
    constructor(left: BindingIdentifier | BindingPattern, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isAssignmentPattern(p: {
    type: StatementType;
}): p is AssignmentPattern;
export declare class AsyncArrowFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: (FunctionParameter | PatternParam)[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(params: (FunctionParameter | PatternParam)[], body: BlockStatement | Expression, expression: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class AsyncFunctionDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class AsyncFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class AwaitExpression implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    /**
     * Used by escodegen.
     */
    delegate?: unknown;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export type BinaryOperator = '+' | '-';
export declare class BinaryExpression implements BaseNode {
    type: StatementType;
    operator: string;
    readonly left: Expression;
    readonly right: Expression;
    constructor(operator: string, left: Expression, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class BlockStatement implements BaseNode {
    readonly type: StatementType;
    readonly body: Statement[] | StatementListItem[];
    constructor(body: Statement[] | StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class BreakStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier | null;
    constructor(label: Identifier | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class CallExpression implements BaseNode {
    type: string;
    callee: Expression | Import;
    arguments: ArgumentListElement[];
    optional: boolean;
    constructor(callee: Expression | Import, args: ArgumentListElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class CatchClause implements BaseNode {
    readonly type: StatementType;
    readonly param: BindingIdentifier | BindingPattern;
    readonly body: BlockStatement;
    /**
     * Used by escodegen.
     */
    guard?: Expression;
    constructor(param: BindingIdentifier | BindingPattern, body: BlockStatement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ClassBody implements BaseNode {
    readonly type: StatementType;
    readonly body: (Property | MethodDefinition)[];
    constructor(body: (Property | MethodDefinition)[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ClassDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly superClass: Identifier | Expression | null;
    readonly body: ClassBody;
    constructor(id: Identifier | null, superClass: Identifier | Expression | null, body: ClassBody);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ClassExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly superClass: Identifier | Expression | null;
    readonly body: ClassBody;
    constructor(id: Identifier | null, superClass: Identifier | Expression | null, body: ClassBody);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class MemberExpression implements BaseNode {
    readonly type: StatementType;
    readonly computed: boolean;
    readonly object: Expression;
    readonly property: Expression;
    constructor(object: Expression, property: Expression, computed: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ComputedMemberExpression extends MemberExpression {
    constructor(object: Expression, property: Expression);
}
export declare class ConditionalExpression implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Expression;
    readonly alternate: Expression;
    constructor(test: Expression, consequent: Expression, alternate: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ContinueStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier | null;
    constructor(label: Identifier | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class DebuggerStatement implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Directive implements BaseNode {
    readonly type: StatementType;
    readonly expression: Expression;
    readonly directive: string;
    /**
     * Used by escodegen.
     */
    raw?: string;
    constructor(expression: Expression, directive: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class DoWhileStatement implements BaseNode {
    readonly type: StatementType;
    body: Statement;
    readonly test: Expression;
    constructor(body: Statement, test: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class EmptyStatement implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ExportAllDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly source: Literal;
    constructor(source: Literal);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ExportDefaultDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declaration: ExportableDefaultDeclaration;
    constructor(declaration: ExportableDefaultDeclaration);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ExportNamedDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declaration: ExportableNamedDeclaration | StatementListItem | null;
    readonly specifiers: ExportSpecifier[];
    readonly source: Literal | null;
    constructor(declaration: ExportableNamedDeclaration | StatementListItem | null, specifiers: ExportSpecifier[], source: Literal | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ExportSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly exported: Identifier;
    readonly local: Identifier;
    constructor(local: Identifier, exported: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ExpressionStatement implements BaseNode {
    readonly type: StatementType;
    readonly expression: Expression;
    constructor(expression: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ForInStatement implements BaseNode {
    readonly type: StatementType;
    readonly left: Expression;
    readonly right: Expression;
    readonly body: Statement | Expression;
    readonly each: boolean;
    constructor(left: Expression, right: Expression, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ForOfStatement implements BaseNode {
    readonly type: StatementType;
    readonly left: Expression;
    readonly right: Expression;
    readonly body: Statement | Expression;
    constructor(left: Expression, right: Expression, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ForStatement implements BaseNode {
    readonly type: StatementType;
    readonly init: Expression | null;
    readonly test: Expression | null;
    readonly update: Expression | null;
    body: Statement | Expression;
    constructor(init: Expression | null, test: Expression | null, update: Expression | null, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class FunctionDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    readonly rest?: Identifier;
    readonly defaults?: Expression[];
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement, generator: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class FunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    readonly rest?: Identifier;
    readonly defaults?: Expression[];
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement | Expression, generator: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Identifier implements BaseNode {
    type: StatementType;
    name: string;
    range: [number, number];
    constructor(name: string | number);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
}
export declare class IfStatement implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Statement;
    readonly alternate: Statement | null;
    constructor(test: Expression, consequent: Statement, alternate: Statement | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Import implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ImportDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly specifiers: ImportDeclarationSpecifier[];
    readonly source: Literal;
    constructor(specifiers: ImportDeclarationSpecifier[], source: Literal);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ImportDefaultSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    constructor(local: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ImportNamespaceSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    constructor(local: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ImportSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    readonly imported: Identifier;
    constructor(local: Identifier, imported: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class LabeledStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier;
    readonly body: Statement | ClassDeclaration;
    constructor(label: Identifier, body: Statement | ClassDeclaration);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Literal implements BaseNode {
    readonly type: StatementType;
    readonly value: boolean | number | string | null;
    readonly raw: string;
    range: [number, number];
    constructor(value: boolean | number | string | null, raw: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
}
export declare class MetaProperty implements BaseNode {
    readonly type: StatementType;
    readonly meta: Identifier;
    readonly property: Identifier;
    constructor(meta: Identifier, property: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class MethodDefinition implements BaseNode {
    readonly type: StatementType;
    readonly key: Expression | null;
    readonly computed: boolean;
    readonly value: AsyncFunctionExpression | FunctionExpression | null;
    readonly kind: string;
    readonly static: boolean;
    constructor(key: Expression | null, computed: boolean, value: AsyncFunctionExpression | FunctionExpression | null, kind: string, isStatic: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Module implements BaseNode {
    readonly type: StatementType;
    readonly body: StatementListItem[];
    readonly sourceType: string;
    comments?: Comment[];
    tokens?: TokenEntry[];
    errors?: Error[];
    constructor(body: StatementListItem[]);
    leadingComments?: Comment[];
    trailingComments?: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class NewExpression implements BaseNode {
    readonly type: StatementType;
    readonly callee: Expression;
    readonly arguments: ArgumentListElement[];
    constructor(callee: Expression, args: ArgumentListElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ObjectExpression implements BaseNode {
    type: StatementType;
    readonly properties: ObjectExpressionProperty[];
    constructor(properties: ObjectExpressionProperty[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isObjectExpression(p: {
    type: StatementType;
}): p is ObjectExpression;
export declare class ObjectPattern implements BaseNode {
    readonly type: StatementType;
    readonly properties: ObjectPatternProperty[];
    constructor(properties: ObjectPatternProperty[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isObjectPattern(p: {
    type: StatementType;
}): p is ObjectPattern;
export declare class Property implements BaseNode {
    readonly type: StatementType;
    readonly key: PropertyKey;
    readonly computed: boolean;
    readonly value: PropertyValue | null;
    readonly kind: string;
    readonly method: boolean;
    readonly shorthand: boolean;
    constructor(kind: 'get' | 'set' | 'init', key: PropertyKey, computed: boolean, value: PropertyValue | null, method: boolean, shorthand: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isProperty(p: {
    type: StatementType;
}): p is Property;
export declare class RegexLiteral implements BaseNode {
    readonly type: StatementType;
    readonly value: RegExp;
    readonly raw: string;
    readonly regex: {
        pattern: string;
        flags: string;
    };
    constructor(value: RegExp, raw: string, pattern: string, flags: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class RestElement implements BaseNode {
    type: StatementType;
    argument: BindingIdentifier | BindingPattern | ReinterpretedExpressionPattern;
    constructor(argument: BindingIdentifier | BindingPattern | ReinterpretedExpressionPattern);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isRestElement(p: {
    type: StatementType;
}): p is RestElement;
export declare class RestProperty implements BaseNode {
    type: StatementType;
    argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isRestProperty(p: {
    type: StatementType;
}): p is RestProperty;
export declare class ReturnStatement implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression | null;
    constructor(argument: Expression | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Script implements BaseNode {
    readonly type: StatementType;
    readonly body: StatementListItem[];
    readonly sourceType: string;
    comments?: Comment[];
    tokens?: TokenEntry[];
    errors?: Error[];
    constructor(body: StatementListItem[]);
    leadingComments?: Comment[];
    trailingComments?: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isProgram(p: {
    type: StatementType;
}): p is Module | Script;
export declare class SequenceExpression implements BaseNode {
    readonly type: StatementType;
    readonly expressions: Expression[];
    constructor(expressions: Expression[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isSequenceExpression(p: {
    type: StatementType;
}): p is SequenceExpression;
export declare class SpreadElement implements BaseNode {
    type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isSpreadElement(p: {
    type: StatementType;
}): p is SpreadElement;
export declare class SpreadProperty implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isSpreadProperty(p: {
    type: StatementType;
}): p is SpreadProperty;
export declare class StaticMemberExpression implements BaseNode {
    readonly type: StatementType;
    readonly computed: boolean;
    readonly object: Expression;
    readonly property: Expression;
    constructor(object: Expression, property: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class Super implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class SwitchCase implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Statement[] | StatementListItem[];
    constructor(test: Expression, consequent: Statement[] | StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class SwitchStatement implements BaseNode {
    readonly type: StatementType;
    readonly discriminant: Expression;
    readonly cases: SwitchCase[];
    constructor(discriminant: Expression, cases: SwitchCase[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class TaggedTemplateExpression implements BaseNode {
    readonly type: StatementType;
    readonly tag: Expression;
    readonly quasi: TemplateLiteral;
    constructor(tag: Expression, quasi: TemplateLiteral);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export interface TemplateElementValue {
    cooked: string;
    raw: string;
}
export declare class TemplateElement implements BaseNode {
    readonly type: StatementType;
    readonly value: TemplateElementValue;
    readonly tail: boolean;
    constructor(value: TemplateElementValue, tail: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class TemplateLiteral implements BaseNode {
    readonly type: StatementType;
    readonly quasis: TemplateElement[];
    readonly expressions: Expression[];
    constructor(quasis: TemplateElement[], expressions: Expression[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ThisExpression implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class ThrowStatement implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class TryStatement implements BaseNode {
    readonly type: StatementType;
    readonly block: BlockStatement;
    readonly handler: CatchClause | null;
    readonly finalizer: BlockStatement | null;
    /**
     * Used by escodegen.
     */
    handlers: CatchClause[];
    /**
     * Used by escodegen.
     */
    guardedHandlers: CatchClause[];
    constructor(block: BlockStatement, handler: CatchClause | null, finalizer: BlockStatement | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class UnaryExpression implements BaseNode {
    readonly type: StatementType;
    operator: string;
    readonly argument: Expression;
    readonly prefix: boolean;
    constructor(operator: string, argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export type UpdateOperator = "++" | "--";
export declare class UpdateExpression implements BaseNode {
    readonly type: StatementType;
    operator: string;
    readonly argument: Expression;
    readonly prefix: boolean;
    constructor(operator: string, argument: Expression, prefix: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class VariableDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declarations: VariableDeclarator[];
    readonly kind: string;
    constructor(declarations: VariableDeclarator[], kind: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isVariableDeclaration(p: {
    type: StatementType;
}): p is VariableDeclaration;
export declare class VariableDeclarator implements BaseNode {
    readonly type: StatementType;
    readonly id: BindingIdentifier | BindingPattern;
    readonly init: Expression | null;
    constructor(id: BindingIdentifier | BindingPattern, init: Expression | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class WhileStatement implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    body: Statement;
    constructor(test: Expression, body: Statement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class WithStatement implements BaseNode {
    readonly type: StatementType;
    readonly object: Expression;
    readonly body: Statement;
    constructor(object: Expression, body: Statement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare class YieldExpression implements BaseNode {
    type: StatementType;
    argument: Expression | null;
    delegate: boolean;
    constructor(argument: Expression | null, delegate: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
export declare function isYieldExpression(p: {
    type: StatementType;
}): p is YieldExpression;
