import { ErrorHandler, Error } from './error-handler';
import { ParseDelegate, ParseOptions } from './esprima';
import { ArgumentListElement, ArrayExpression, ArrayPattern, AssignmentPattern, AsyncFunctionDeclaration, AsyncFunctionExpression, AwaitExpression, BaseNode, BaseNodeWithoutComments, BindingIdentifier, BindingPattern, BlockStatement, BreakStatement, CatchClause, ClassBody, ClassDeclaration, ClassExpression, ContinueStatement, DebuggerStatement, Directive, DoWhileStatement, EmptyStatement, ExportDeclaration, ExportSpecifier, Expression, ExpressionStatement, ForInStatement, ForOfStatement, ForStatement, FunctionDeclaration, FunctionExpression, FunctionParameter, Identifier, IfStatement, Import, ImportDeclaration, ImportDefaultSpecifier, ImportNamespaceSpecifier, ImportSpecifier, LabeledStatement, Literal, MetaProperty, MethodDefinition, Module, NewExpression, ObjectExpression, ObjectPattern, PatternParam, Property, PropertyKey, ReinterpretedExpressionPattern, RestElement, RestProperty, ReturnStatement, Script, SequenceExpression, SpreadElement, SpreadProperty, Statement, StatementListItem, Super, SwitchCase, SwitchStatement, TemplateElement, TemplateLiteral, ThrowStatement, TryStatement, VariableDeclaration, VariableDeclarator, WhileStatement, WithStatement, YieldExpression } from './nodes';
import { Scanner } from './scanner';
import { RawToken, RawTokenValue, TokenEntry } from './token';
interface Config {
    range: boolean;
    loc: boolean;
    source: string | null;
    tokens: boolean;
    comment: boolean;
    tolerant: boolean;
}
interface Context {
    isModule: boolean;
    allowIn: boolean;
    allowStrictDirective: boolean;
    allowYield: boolean;
    await: boolean;
    firstCoverInitializedNameError: RawToken | null;
    isAssignmentTarget: boolean;
    isBindingElement: boolean;
    inFunctionBody: boolean;
    inIteration: boolean;
    inSwitch: boolean;
    labelSet: {
        [key: string]: boolean;
    };
    strict: boolean;
}
export interface Marker {
    index: number;
    line: number;
    column: number;
}
interface DeclarationOptions {
    inFor: boolean;
}
export interface CanBeParser {
    readonly config: Config;
    tokens: TokenEntry[];
    readonly errorHandler: ErrorHandler;
    parseModule(): Module;
    parseScript(): Script;
}
export declare class Parser implements CanBeParser {
    readonly config: Config;
    readonly delegate: ParseDelegate | null;
    readonly errorHandler: ErrorHandler;
    readonly scanner: Scanner;
    readonly operatorPrecedence: {
        [ch: string]: number;
    };
    lookahead: RawToken;
    hasLineTerminator: boolean;
    context: Context;
    tokens: TokenEntry[];
    startMarker: Marker;
    lastMarker: Marker;
    constructor(code: string, options: ParseOptions, delegate: ParseDelegate | null);
    throwError(messageFormat: string, ..._values: any[]): never;
    tolerateError(messageFormat: string, ..._values: any[]): void;
    unexpectedTokenError(token?: RawToken, message?: string): Error;
    throwUnexpectedToken(token?: RawToken, message?: string): never;
    tolerateUnexpectedToken(token?: RawToken, message?: string): void;
    collectComments(): void;
    getTokenRaw(token: RawToken): string;
    convertToken(token: RawToken): TokenEntry;
    nextToken(): RawToken;
    nextRegexToken(): RawToken;
    /**
     * Creates a Marker reflecting the current index, line, and column.
     */
    createMarker(): Readonly<Marker>;
    startNode(token: Readonly<RawToken>): Readonly<Marker>;
    finalize<N extends BaseNodeWithoutComments>(marker: Readonly<Marker>, node: N): N;
    expect(value: RawTokenValue): void;
    expectCommaSeparator(): void;
    expectKeyword(keyword: 'break' | 'case' | 'catch' | 'class' | 'continue' | 'debugger' | 'do' | 'export' | 'finally' | 'for' | 'function' | 'if' | 'import' | 'return' | 'super' | 'switch' | 'throw' | 'try' | 'var' | 'while' | 'with' | 'yield'): void;
    match(value: string): boolean;
    matchKeyword(keyword: string): boolean;
    matchContextualKeyword(keyword: string): boolean;
    matchAssign(): boolean;
    isolateCoverGrammar(parseFunction: () => Expression): Expression;
    inheritCoverGrammar(parseFunction: () => Expression): Expression;
    consumeSemicolon(): void;
    parsePrimaryExpression(): Expression;
    parseSpreadElement(): SpreadElement;
    parseArrayInitializer(): ArrayExpression;
    parsePropertyMethod(params: {
        simple: boolean;
        firstRestricted?: RawToken;
        message?: string;
        stricted?: RawToken;
    }): Expression;
    parsePropertyMethodFunction(): FunctionExpression;
    parsePropertyMethodAsyncFunction(): FunctionExpression;
    parseObjectPropertyKey(): PropertyKey;
    /**
     * TODO: PropertyKey has been extended to Expression so we are missing something here.
     * @param key
     * @param value
     * @returns
     */
    isPropertyKey(key: PropertyKey, value: string): boolean;
    parseObjectProperty(hasProto: {
        value: boolean;
    }): Property;
    parseSpreadProperty(): SpreadProperty;
    parseObjectInitializer(): ObjectExpression;
    parseTemplateHead(): TemplateElement;
    parseTemplateElement(): TemplateElement;
    parseTemplateLiteral(): TemplateLiteral;
    /**
     * The argument is being mutated, which is tricky from a type perspective.
     */
    reinterpretExpressionAsPattern(expr: Expression): ReinterpretedExpressionPattern;
    parseGroupExpression(): Expression;
    parseArguments(): ArgumentListElement[];
    isIdentifierName(token: Readonly<RawToken>): boolean;
    parseIdentifierName(): Identifier;
    parseNewExpression(): MetaProperty | NewExpression;
    parseAsyncArgument(): Expression;
    parseAsyncArguments(): ArgumentListElement[];
    matchImportCall(): boolean;
    parseImportCall(): Import;
    parseLeftHandSideExpressionAllowCall(): Expression;
    parseSuper(): Super;
    parseLeftHandSideExpression(): Expression;
    parseUpdateExpression(): Expression;
    parseAwaitExpression(): AwaitExpression;
    parseUnaryExpression(): Expression;
    parseExponentiationExpression(): Expression;
    binaryPrecedence(token: RawToken): number;
    parseBinaryExpression(): Expression;
    parseConditionalExpression(): Expression;
    checkPatternParam(options: {
        simple?: boolean;
        stricted?: RawToken;
    }, param: PatternParam): void;
    reinterpretAsCoverFormalsList(expr: BaseNode): {
        simple: boolean;
        params: PatternParam[];
        stricted: RawToken;
        firstRestricted: RawToken;
        message: string;
    };
    parseAssignmentExpression(): Expression;
    parseExpression(): Expression | SequenceExpression;
    parseStatementListItem(): StatementListItem;
    parseBlock(): BlockStatement;
    parseLexicalBinding(kind: 'const' | 'let', options: {
        inFor?: boolean;
    }): VariableDeclarator;
    parseBindingList(kind: 'const' | 'let', options: {}): VariableDeclarator[];
    isLexicalDeclaration(): boolean;
    parseLexicalDeclaration(options: {}): VariableDeclaration;
    parseBindingRestElement(params: RawToken[], kind?: 'const' | 'let' | 'var'): RestElement;
    parseArrayPattern(params: RawToken[], kind?: 'const' | 'let' | 'var'): ArrayPattern;
    parsePropertyPattern(params: RawToken[], kind?: 'const' | 'let' | 'var'): Property;
    parseRestProperty(params: RawToken[], _kind?: 'const' | 'let' | 'var'): RestProperty;
    parseObjectPattern(params: RawToken[], kind?: 'const' | 'let' | 'var'): ObjectPattern;
    parsePattern(params: RawToken[], kind?: 'const' | 'let' | 'var'): BindingIdentifier | BindingPattern;
    parsePatternWithDefault(params: RawToken[], kind?: 'const' | 'let' | 'var'): AssignmentPattern | BindingIdentifier | BindingPattern;
    parseVariableIdentifier(kind?: 'const' | 'let' | 'var'): Identifier;
    parseVariableDeclaration(options: DeclarationOptions): VariableDeclarator;
    parseVariableDeclarationList(options: {
        inFor: boolean;
    }): VariableDeclarator[];
    parseVariableStatement(): VariableDeclaration;
    parseEmptyStatement(): EmptyStatement;
    parseExpressionStatement(): ExpressionStatement;
    parseIfClause(): Statement;
    parseIfStatement(): IfStatement;
    parseDoWhileStatement(): DoWhileStatement;
    parseWhileStatement(): WhileStatement;
    parseForStatement(): ForStatement | ForInStatement | ForOfStatement;
    parseContinueStatement(): ContinueStatement;
    parseBreakStatement(): BreakStatement;
    parseReturnStatement(): ReturnStatement;
    parseWithStatement(): WithStatement;
    parseSwitchCase(): SwitchCase;
    parseSwitchStatement(): SwitchStatement;
    parseLabelledStatement(): LabeledStatement | ExpressionStatement;
    parseThrowStatement(): ThrowStatement;
    parseCatchClause(): CatchClause;
    parseFinallyClause(): BlockStatement;
    parseTryStatement(): TryStatement;
    parseDebuggerStatement(): DebuggerStatement;
    parseStatement(): Statement;
    parseFunctionSourceElements(): BlockStatement;
    validateParam(options: {
        stricted?: RawToken | Identifier;
        message?: string;
        firstRestricted?: RawToken | Identifier;
        paramSet?: {
            [key: string]: boolean;
        };
    }, param: RawToken | Identifier, name: RawTokenValue): void;
    parseRestElement(params: RawToken[]): RestElement;
    parseFormalParameter(options: {
        simple: boolean;
        stricted?: RawToken;
        params: (AssignmentPattern | BindingPattern | Identifier | RestElement)[];
    }): void;
    parseFormalParameters(firstRestricted?: RawToken): {
        simple: boolean;
        params: FunctionParameter[];
        stricted: RawToken;
        firstRestricted: RawToken;
        message: string;
    };
    matchAsyncFunction(): boolean;
    parseFunctionDeclaration(identifierIsOptional?: boolean): AsyncFunctionDeclaration | FunctionDeclaration;
    parseFunctionExpression(): AsyncFunctionExpression | FunctionExpression;
    parseDirective(): Directive | ExpressionStatement;
    parseDirectivePrologues(): Statement[];
    qualifiedPropertyName(token: RawToken): boolean;
    parseGetterMethod(): FunctionExpression;
    parseSetterMethod(): FunctionExpression;
    parseGeneratorMethod(): FunctionExpression;
    isStartOfExpression(): boolean;
    parseYieldExpression(): YieldExpression;
    parseClassElement(hasConstructor: {
        value: boolean;
    }): Property | MethodDefinition;
    parseClassElementList(): (Property | MethodDefinition)[];
    parseClassBody(): ClassBody;
    parseClassDeclaration(identifierIsOptional?: boolean): ClassDeclaration;
    parseClassExpression(): ClassExpression;
    parseModule(): Module;
    parseScript(): Script;
    parseModuleSpecifier(): Literal;
    parseImportSpecifier(): ImportSpecifier;
    parseNamedImports(): ImportSpecifier[];
    parseImportDefaultSpecifier(): ImportDefaultSpecifier;
    parseImportNamespaceSpecifier(): ImportNamespaceSpecifier;
    parseImportDeclaration(): ImportDeclaration;
    parseExportSpecifier(): ExportSpecifier;
    parseExportDeclaration(): ExportDeclaration;
}
export {};
