import { GenerateOptions } from './ecmascript/index';
import { IfStatement, ParseOptions, VariableDeclaration } from './ecmascript/index';
/**
 * Provides the MathScript module
 */
/**
 * Determines the transformation action for operators.
 */
export interface OperatorTransform {
    /**
     * The nature of the transformation.
     *
     * 'rename' replaces the name of the operator with the value property.
     *
     * 'function' replaces the operator with a function call with a name given by the value property.
     *
     * 'error' causes an exception to be raised if the operator is detected. The value property becomes the error message.
     */
    kind: 'rename' | 'function' | 'error';
    /**
     * The transformed value. How it is used depends on the kind property.
     *
     * 'rename': value is treated as the new operator name.
     * 'function': value is treated as the new function name.
     * 'error': value is treated as the error message.
     */
    value: string;
}
/**
 * The combined parsing and transpilation customization.
 * These properties affect the parsing and AST transformations before code generation.
 */
export interface TranspileSettings extends ParseOptions {
    /**
     * Specifies the timeout value in milliseconds if the code has loop checks inserted.
     */
    timeout?: number;
    /**
     * Used to determine whether timeout checks will be added to the code to prevent infinite loops.
     */
    noLoopCheck?: boolean;
    /**
     * The namespace qualification to use for replacements of operators with functions.
     */
    namespace?: string;
    /**
     * The replacements of binary operators that will be made.
     * The name key is used to match the operator symbol.
     * The mapped transform is used to specify the action to be performed.
     */
    binOp: {
        [name: string]: OperatorTransform;
    };
    /**
     * The replacements of unary operators that will be made.
     * The name key is used to match the operator symbol.
     * The mapped transform is used to specify the action to be performed.
     */
    unaryOp: {
        [name: string]: OperatorTransform;
    };
}
/**
 * The combined parsing and transpilation customization.
 * These properties affect the parsing and AST transformations before code generation.
 */
export interface TranspileOptions extends ParseOptions {
    /**
     * Specifies the timeout value in milliseconds if the code has loop checks inserted.
     */
    timeout?: number;
    /**
     * Used to determine whether timeout checks will be added to the code to prevent infinite loops.
     */
    noLoopCheck?: boolean;
    /**
     * The namespace qualification to use for replacements of operators with functions.
     */
    namespace?: string;
    /**
     * The replacements of binary operators that will be made.
     * The name key is used to match the operator symbol.
     * The mapped transform is used to specify the action to be performed.
     */
    binOp?: {
        [name: string]: OperatorTransform;
    };
    /**
     * The replacements of unary operators that will be made.
     * The name key is used to match the operator symbol.
     * The mapped transform is used to specify the action to be performed.
     */
    unaryOp?: {
        [name: string]: OperatorTransform;
    };
}
/**
 * Transpiles JavaScript Expression syntax to "MathScript", a similar syntax to JavaScript but perhaps with different operator symbols.
 * Examples of MathScript(s) include Algebrite (a symbolic algebra library), and JessieCode (a scripting language for JSXGraph).
 * Typical use cases include renaming operators, converting operators to function calls, and rejecting operators as syntax errors.
 * @param code The source code to be transpiled.
 * @param transpileOptions The options for both parsing and mutation.
 * @param generateOptions The options for code generation.
 */
export declare function jsToMathScript(code: string, transpileOptions?: TranspileOptions, generateOptions?: GenerateOptions): {
    code: string;
};
/**
 * Generate 2 ASTs for the code to be inserted in loops for infinite run protection.
 */
export declare function getLoopProtectorBlocks(varName: string, timeout: number): {
    before: VariableDeclaration;
    inside: IfStatement;
};
export declare function generateRandomId(length?: number): string;
